@@ -123,6 +123,17 @@ new_fluid_oss_audio_driver(fluid_settings_t *settings, fluid_synth_t *synth)
     char *devname = NULL;
     int format;
 
+    FLUID_LOG(FLUID_WARN,
+              "\n\n"
+              "================= OSS audio driver has been deprecated! ==================\n"
+              "You're using the OSS driver. This driver is old, unmaintained and believed\n"
+              "to be unused. If you still need it, pls. let us know by posting to our\n"
+              "mailing list at fluid-dev@nongnu.org - otherwise this driver might be removed\n"
+              "in a future release of FluidSynth!\n"
+              "================= OSS audio driver has been deprecated! ==================\n"
+              "\n"
+    );
+
     dev = FLUID_NEW(fluid_oss_audio_driver_t);
 
     if(dev == NULL)
@@ -296,6 +307,17 @@ new_fluid_oss_audio_driver2(fluid_settings_t *settings, fluid_audio_func_t func,
     int realtime_prio = 0;
     int format;
 
+    FLUID_LOG(FLUID_WARN,
+              "\n\n"
+              "================= OSS audio driver has been deprecated! ==================\n"
+              "You're using the OSS driver. This driver is old, unmaintained and believed\n"
+              "to be unused. If you still need it, pls. let us know by posting to our\n"
+              "mailing list at fluid-dev@nongnu.org - otherwise this driver might be removed\n"
+              "in a future release of FluidSynth!\n"
+              "================= OSS audio driver has been deprecated! ==================\n"
+              "\n"
+    );
+
     dev = FLUID_NEW(fluid_oss_audio_driver_t);
 
     if(dev == NULL)
@@ -607,6 +629,17 @@ new_fluid_oss_midi_driver(fluid_settings_t *settings,
     int realtime_prio = 0;
     char *device = NULL;
 
+    FLUID_LOG(FLUID_WARN,
+              "\n\n"
+              "================= OSS MIDI driver has been deprecated! ==================\n"
+              "You're using the OSS driver. This driver is old, unmaintained and believed\n"
+              "to be unused. If you still need it, pls. let us know by posting to our\n"
+              "mailing list at fluid-dev@nongnu.org - otherwise this driver might be removed\n"
+              "in a future release of FluidSynth!\n"
+              "================= OSS MIDI driver has been deprecated! ==================\n"
+              "\n"
+    );
+
     /* not much use doing anything */
     if(handler == NULL)
     {
